


[BITS 16]

boot:

    ; Print greeting string
    push greeting
    call putStr
    add sp,2

.lp:
    in ax,0x64          ; Get keyboard status reg
    and ax,1
    je .lp              ; If LSB in status is clear, no key
    in al,0x60          ; Get scancode
    mov ah,0
    push ax
    call print_hex_16   ; Print the scancode
    add sp,2
    jmp .lp             ; Check for another keypress



; Prints a 16-bit hex value passed on the stack
print_hex_16:
    push bp   ; Preamble
    mov bp,sp
    push ax
    push bx
    push cx

    mov cx,16 ; Load the count register, which tracks how many bits to shift

print_hex_loop:
    mov ax,[4+bp] ; Get number to print in AX
    sub cx,4      ; Decrement the shift count
    shr ax,cl     ; Shift the number
    and ax,0xf    ; Mask off higher order bits
    add al,'0'    ; Add ASCII offset
    cmp al,'9'
    jle do_print_hex
    add al,7
do_print_hex:
    mov ah, 0x0e  ; Call BIOS to print character
    xor bh,bh
    mov bl, 0x07
    int 16
    cmp cx,0
    jg print_hex_loop

    pop cx        ; Collapse stack frame
    pop bx
    pop ax
    pop bp
    ret

; putStr
;
; Prints a NULL-terminated string to the console by calling BIOS Int 0x10
;
; |-------------------------------|
; |          Ptr To String        |
; |-------------------------------|
; |         Return Address        |
; |-------------------------------|
; |          Caller's BP          |
; |-------------------------------|
;
putStr:
    push bp
    mov bp,sp
    push ax
    push si

    ; Load the address of string to print into SI
    mov si,[4+BP]

    ; Set up the registers for a BIOS call to print
    mov ah, 0x0e
    xor bh,bh
    mov bl,7
put_str_loop:
    mov al,[ds:si]
    cmp al,0
    je put_str_done
    int 16
    inc si
    jmp put_str_loop

put_str_done:
    pop si
    pop ax
    pop bp
    ret



; Data
greeting db 'Bootloader starting...',0xd,0xa,0

crlfstr  db 0xd,0xa,0
errmsg   db 'ERROR reading sector',0xd,0xa,0
key_pressed_msg db '   Key pressed!',0xd,0xa,0
num_key_presses:
    dw 0

; Keyboard scancode table
; https://stackoverflow.com/questions/51693306/registering-interrupt-in-16-bit-x86-assembly
; Scancode to ASCII character translation table
keyboard_map:
    db  0,  27, '1', '2', '3', '4', '5', '6', '7', '8'    ; 9
    db '9', '0', '-', '=', 0x08                           ; Backspace
    db 0x09                                               ; Tab
    db 'q', 'w', 'e', 'r'                                 ; 19
    db 't', 'y', 'u', 'i', 'o', 'p', '[', ']', 0x0a       ; Enter key
    db 0                                                  ; 29   - Control
    db 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';'   ; 39
    db "'", '`', 0                                        ; Left shift
    db "\", 'z', 'x', 'c', 'v', 'b', 'n'                  ; 49
    db 'm', ',', '.', '/', 0                              ; Right shift
    db '*'
    db 0                                                  ; Alt
    db ' '                                                ; Space bar
    db 0                                                  ; Caps lock
    db 0                                                  ; 59 - F1 key ... >
    db 0,   0,   0,   0,   0,   0,   0,   0
    db 0                                                  ; < ... F10
    db 0                                                  ; 69 - Num lock
    db 0                                                  ; Scroll Lock
    db 0                                                  ; Home key
    db 0                                                  ; Up Arrow
    db 0                                                  ; Page Up
    db '-'
    db 0                                                  ; Left Arrow
    db 0
    db 0                                                  ; Right Arrow
    db '+'
    db 0                                                  ; 79 - End key
    db 0                                                  ; Down Arrow
    db 0                                                  ; Page Down
    db 0                                                  ; Insert Key
    db 0                                                  ; Delete Key
    db 0,   0,   0
    db 0                                                  ; F11 Key
    db 0                                                  ; F12 Key
    times 128 - ($-keyboard_map) db 0                     ; All other keys are undefined



times 446-($-$$) db 0 ; Empty space btw bootloader and partition table

; Partition table
;                 Status  Head Cylinder:Sector   Cylinder  Type  Head
partitionEntry1 db 0x80,  0x00,      0x02,         0x00,   0x83, 0x08
;                 Cylinder:Sector  Cylinder        1st LBA    Last LBA
                db      0x08,       0x00,    0x01, 0, 0, 0, 0xff, 0x01,0,0
times 510-($-$$) db 0

db 0x55 ; Magic Numbers
db 0xaa ; To make disk bootable


